Package: spsurvey
Title: Spatial Sampling Design and Analysis
Version: 5.6.1
Authors@R: c(
    person("Michael", "Dumelle", role=c("aut","cre"),
           email = "Dumelle.Michael@epa.gov", comment = c(ORCID = "0000-0002-3393-5529")),
    person("Tom", "Kincaid", role="aut"),
    person("Anthony (Tony) R.", "Olsen",  role="aut"),
    person("Marc", "Weber",  role="aut"),
    person("Don", "Stevens",  role="ctb"),
    person("Denis", "White",  role="ctb"),
    person("Amanda M.", "Nahlik",  role="ctb"),
    person("Sarah", "Lehmann",  role="ctb")
    )
Maintainer: Michael Dumelle <Dumelle.Michael@epa.gov>
Description: A design-based approach to statistical inference, with a focus on spatial data. Spatially balanced samples are selected using the Generalized Random Tessellation Stratified (GRTS) algorithm. The GRTS algorithm can be applied to finite resources (point geometries) and infinite resources (linear / linestring and areal / polygon geometries) and flexibly accommodates a diverse set of sampling design features, including stratification, unequal inclusion probabilities, proportional (to size) inclusion probabilities, legacy (historical) sites, a minimum distance between sites, and two options for replacement sites (reverse hierarchical order and nearest neighbor). Data are analyzed using a wide range of analysis functions that perform categorical variable analysis, continuous variable analysis, attributable risk analysis, risk difference analysis, relative risk analysis, change analysis, and trend analysis. spsurvey can also be used to summarize objects, visualize objects, select samples that are not spatially balanced, select panel samples, measure the amount of spatial balance in a sample, adjust design weights, and more. For additional details, see Dumelle et al. (2023) <doi:10.18637/jss.v105.i03>.
Depends: R (>= 3.5.0), sf, survey (>= 4.1-1)
Imports: boot, crossdes, deldir, graphics, grDevices, lme4, MASS,
        sampling, stats, units
Suggests: knitr, testthat, rmarkdown
License: GPL (>= 3)
URL: https://usepa.github.io/spsurvey/,
        https://github.com/USEPA/spsurvey
BugReports: https://github.com/USEPA/spsurvey/issues
VignetteBuilder: knitr
Encoding: UTF-8
LazyData: true
RoxygenNote: 7.3.2
NeedsCompilation: no
Packaged: 2026-05-04 19:25:28 UTC; MDUMELLE
Author: Michael Dumelle [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-3393-5529>),
  Tom Kincaid [aut],
  Anthony (Tony) R. Olsen [aut],
  Marc Weber [aut],
  Don Stevens [ctb],
  Denis White [ctb],
  Amanda M. Nahlik [ctb],
  Sarah Lehmann [ctb]
Repository: CRAN
Date/Publication: 2026-05-04 21:10:02 UTC
Built: R 4.6.0; ; 2026-05-05 00:13:35 UTC; unix
