# Generated by roxygen2: do not edit by hand

S3method(plot,sp_CDF)
S3method(plot,sp_design)
S3method(plot,sp_frame)
S3method(print,sp_design)
S3method(print,sp_summary.sp_design)
S3method(print,sp_summary.sp_frame)
S3method(print,summary.sp_design)
S3method(print,summary.sp_frame)
S3method(sp_plot,default)
S3method(sp_plot,sp_design)
S3method(sp_summary,default)
S3method(sp_summary,sp_design)
S3method(summary,sp_design)
S3method(summary,sp_frame)
export(adjwgt)
export(adjwgtNR)
export(ash1_wgt)
export(attrisk_analysis)
export(cat_analysis)
export(cdf_plot)
export(change_analysis)
export(cont_analysis)
export(cont_cdfplot)
export(cont_cdftest)
export(cov_panel_dsgn)
export(diffrisk_analysis)
export(errorprnt)
export(grts)
export(irs)
export(localmean_cov)
export(localmean_var)
export(localmean_weight)
export(pd_summary)
export(power_dsgn)
export(ppd_plot)
export(relrisk_analysis)
export(revisit_bibd)
export(revisit_dsgn)
export(revisit_rand)
export(sp_balance)
export(sp_frame)
export(sp_plot)
export(sp_rbind)
export(sp_summary)
export(sp_unframe)
export(stopprnt)
export(trend_analysis)
export(warnprnt)
import(sf)
import(survey)
importFrom(MASS,ginv)
importFrom(boot,boot)
importFrom(crossdes,find.BIB)
importFrom(deldir,deldir)
importFrom(deldir,tile.list)
importFrom(grDevices,graphics.off)
importFrom(grDevices,pdf)
importFrom(grDevices,rainbow)
importFrom(graphics,axis)
importFrom(graphics,box)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,mtext)
importFrom(graphics,par)
importFrom(graphics,plot)
importFrom(graphics,points)
importFrom(graphics,text)
importFrom(graphics,title)
importFrom(lme4,VarCorr)
importFrom(lme4,lmer)
importFrom(sampling,UPpivotal)
importFrom(stats,addmargins)
importFrom(stats,chisq.test)
importFrom(stats,confint)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,extractAIC)
importFrom(stats,ftable)
importFrom(stats,lm)
importFrom(stats,median)
importFrom(stats,model.frame)
importFrom(stats,model.matrix)
importFrom(stats,na.omit)
importFrom(stats,na.pass)
importFrom(stats,optimize)
importFrom(stats,pchisq)
importFrom(stats,pf)
importFrom(stats,pnorm)
importFrom(stats,qnorm)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,terms)
importFrom(stats,update)
importFrom(stats,var)
importFrom(stats,weights)
importFrom(units,set_units)
